@@ -16,7 +16,17 @@ impl reject::Reject for Forbidden {}
 
 /// Maps to an HTTP 400.
 #[derive(Debug)]
-pub struct BadRequest;
+pub struct BadRequest {
+    message: String,
+}
+
+impl<E: ToString> From<E> for BadRequest {
+    fn from(e: E) -> Self {
+        BadRequest {
+            message: e.to_string(),
+        }
+    }
+}
 
 impl reject::Reject for BadRequest {}
 
@@ -50,15 +60,21 @@ pub async fn recover(err: warp::Rejection) -> Result<impl warp::Reply, Infallibl
     if let Some(Forbidden) = err.find() {
         tracing::info!("rejection: Forbidden");
         Ok(reply::error(MESSAGE_FORBIDDEN, StatusCode::FORBIDDEN))
-    } else if let Some(BadRequest) = err.find() {
+    } else if let Some(BadRequest { message }) = err.find() {
         tracing::info!("rejection: BadRequest");
-        Ok(reply::error(MESSAGE_BAD_REQUEST, StatusCode::BAD_REQUEST))
+        Ok(reply::error(
+            format!("{}: {}", MESSAGE_BAD_REQUEST, message),
+            StatusCode::BAD_REQUEST,
+        ))
     } else if let Some(InternalServerError { message }) = err.find() {
         tracing::error!("error: {}", message);
         Ok(reply::error(message, StatusCode::INTERNAL_SERVER_ERROR))
     } else if let Some(e) = err.find::<warp::filters::body::BodyDeserializeError>() {
         tracing::info!(info = ?e.to_string(), "rejection: BadRequest");
-        Ok(reply::error(MESSAGE_BAD_REQUEST, StatusCode::BAD_REQUEST))
+        Ok(reply::error(
+            format!("{}: {}", MESSAGE_BAD_REQUEST, &e.to_string()),
+            StatusCode::BAD_REQUEST,
+        ))
     } else {
         tracing::error!("unhandled rejection: {:?}", err);
         Ok(reply::error(
